@@ -3,6 +3,8 @@ class ToothpickError(Exception):
 
 class RecordInvalidError(ToothpickError): pass
 
+class ConnectionError(ToothpickError): pass
+
 
 # HTTP response errors
 class ResponseError(ToothpickError):
@@ -59,3 +61,5 @@ class ServerError(ResponseError):
 class TooManyRecordsError(ToothpickError):
     pass
 
+class ForeignKeyError(ToothpickError):
+    pass
