@@ -8,10 +8,12 @@ import os
 
 try:
     import requests
+    from requests.exceptions import ConnectionError as RequestsConnectionError
 except ImportError:
     pass
 try:
     import pymongo as mongo
+    from pymongo.errors import ConnectionFailure as MongoConnectionFailure
 except ImportError:
     pass
 
@@ -50,6 +52,7 @@ class Resource(object):
     def __init__(self, resource_name, config, proxies, options=None):
         self.resource_name = resource_name
         self.config = config
+        self.model_class = None
 
         if proxies == toothpick.DEFAULT:
             proxies = {}
@@ -60,9 +63,12 @@ class Resource(object):
         self.options = options
 
     def __repr__(self):
-        return "%s(%s)[%s]" % (
-            self.__class__.__name__,
-            repr(self.config),
+        if self.model_class:
+            front = self.model_class.__name__
+        else:
+            front = "%s(%s)" % (self.__class__.__name__, repr(self.config))
+        return "%s[%s]" % (
+            front,
             self.resource_name
         )
 
@@ -91,7 +97,7 @@ class Resource(object):
         """
         Decorator for use on model classes.  This method instantiates a
         resource using the given parameters and registers it to the class that
-        is decorated.  
+        is decorated.
         """
 
         def wrap(model_class):
@@ -103,6 +109,7 @@ class Resource(object):
                 **data
             )
             model_class._resources[resource_name] = resource
+            resource.model_class = model_class
 
             return model_class
 
@@ -141,18 +148,23 @@ class RESTfulResource(Resource):
         self.path = path
 
     def url(self, query):
-        if '%' in self.path:
+        path = self.path
+
+        if callable(path):
+            path = path(query)
+
+        if '%' in path:
             try:
                 # query can be a tuple, if path has multiple interpolations
-                full_query = self.path % query
+                full_query = path % query
             except TypeError, e:
                 raise exceptions.ToothpickError("\n".join(
                     ["Wrong number of arguments for resource:",
-                     "  path: %s" % repr(self.path),
+                     "  path: %s" % repr(path),
                      "  query: %s" % repr(query)]
                 ))
         else:
-            full_query = self.path
+            full_query = path
         return "/".join([self.config.base_url, full_query])
 
     def create(self, query, attributes, options=None):
@@ -161,25 +173,33 @@ class RESTfulResource(Resource):
             headers.update(option('headers', options))
 
         body = self.serialize(attributes)
-        response = requests.post(self.url(query),
-                                 data=body,
-                                 headers=headers,
-                                 auth=self.auth())
+        try:
+            response = requests.post(self.url(query),
+                                     data=body,
+                                     headers=headers,
+                                     auth=self.auth())
 
-        self.handle_response(response)
-        return self.deserialize(response.content)
+            self.handle_response(response)
+            return self.deserialize(response.content)
+
+        except RequestsConnectionError, e:
+            raise exceptions.ConnectionError(e)
 
     def read(self, query, options=None):
         headers = self.default_read_headers()
         if option('headers', options):
             headers.update(option('headers', options))
 
-        response = requests.get(self.url(query),
-                                headers=headers, 
-                                auth=self.auth())
+        try:
+            response = requests.get(self.url(query),
+                                    headers=headers,
+                                    auth=self.auth())
 
-        self.handle_response(response)
-        return self.deserialize(response.content)
+            self.handle_response(response)
+            return self.deserialize(response.content)
+
+        except RequestsConnectionError, e:
+            raise exceptions.ConnectionError(e)
 
     def update(self, query, attributes, options=None):
         headers = self.default_update_headers()
@@ -187,43 +207,53 @@ class RESTfulResource(Resource):
             headers.update(option('headers', options))
 
         body = self.serialize(attributes)
-        response = requests.put(self.url(query),
-                                data=body,
-                                headers=headers,
-                                auth=self.auth())
+        try:
+            response = requests.put(self.url(query),
+                                    data=body,
+                                    headers=headers,
+                                    auth=self.auth())
 
-        self.handle_response(response)
-        return attributes
+            self.handle_response(response)
+            return attributes
+
+        except RequestsConnectionError, e:
+            raise exceptions.ConnectionError(e)
 
     def delete(self, query, options=None):
         headers = self.default_delete_headers()
         if option('headers', options):
             headers.update(option('headers', options))
 
-        response = requests.delete(self.url(query),
-                                   headers=headers, 
-                                   auth=self.auth())
+        try:
+            response = requests.delete(self.url(query),
+                                       headers=headers,
+                                       auth=self.auth())
 
-        self.handle_response(response)
-        return
+            self.handle_response(response)
+            return
+
+        except RequestsConnectionError, e:
+            raise exceptions.ConnectionError(e)
 
     def exists(self, query, options = None):
         headers = self.default_exists_headers()
         if option('headers', options):
             headers.update(option('headers', options))
 
-        response = requests.head(self.url(query),
-                                 headers=headers, 
-                                 auth=self.auth())
-
         try:
+            response = requests.head(self.url(query),
+                                     headers=headers,
+                                     auth=self.auth())
+
             self.handle_response(response)
             # TODO: what if the response is an empty list? - not done yet
             return True
+        except RequestsConnectionError, e:
+            raise exceptions.ConnectionError(e)
         except exceptions.ResponseError, e:
             return False
 
-    
+
     def handle_response(self, response):
         '''
         Takes a response object, and either returns it, or raises
@@ -320,9 +350,19 @@ class JSONResource(JSONSerializer, RESTfulResource):
     def default_create_headers(self):
         return {'content-type': 'application/json'}
 
+    def default_read_headers(self):
+        return {'content-type': 'application/json'}
+
     def default_update_headers(self):
         return {'content-type': 'application/json'}
 
+    def default_delete_headers(self):
+        return {'content-type': 'application/json'}
+
+    def default_exists_headers(self):
+        return {'content-type': 'application/json'}
+
+
 class TSVResource(RESTfulResource):
 
     data_delimiter = '\t'
@@ -330,10 +370,10 @@ class TSVResource(RESTfulResource):
 
     def serialize(self, data):
         ''' Turn the given list of dictionaries into a TSV string, with
-        each given dictionary translating to one row of TSV data. Note 
+        each given dictionary translating to one row of TSV data. Note
         that any information about column ordering contained in the original
         TSV string is lost. '''
-        
+
         if type(data) is dict:
             data = [data]
 
@@ -358,8 +398,8 @@ class TSVResource(RESTfulResource):
         return self.line_delimiter.join(rows)
 
     def deserialize(self, data):
-        ''' Turn TSV data into a list of objects, properties defined by the 
-        first row in the data.  If the data has fewer than two lines, 
+        ''' Turn TSV data into a list of objects, properties defined by the
+        first row in the data.  If the data has fewer than two lines,
         we didn't get any objects back and will return []. '''
 
         items = []
@@ -381,7 +421,7 @@ class TSVResource(RESTfulResource):
                     continue
                 # If row[i] is null, should we not set the property?
                 # Or should we set it to None? Or to ""? Same questions
-                # for cases where this row has fewer elements than the 
+                # for cases where this row has fewer elements than the
                 # properties row.
                 item[key] = row[i] if i < len(row) else ""
             items.append(item)
@@ -404,16 +444,19 @@ class MongoResource(Resource):
         self.queryify = bson
         if self.queryify == toothpick.DEFAULT or not self.queryify:
             self.queryify = lambda x:x
-        
+
         self._client = None # initialized at time of first query
 
     @property
     def client(self):
-        if not self._client:
-            self._client = mongo.MongoClient(self.config.host,  
-                                             self.config.port,  
-                                             auto_start_request=False)
-        return self._client
+        try:
+            if not self._client:
+                self._client = mongo.MongoClient(self.config.host,
+                                                 self.config.port,
+                                                 auto_start_request=False)
+            return self._client
+        except MongoConnectionFailure, e:
+            raise exceptions.ConnectionError(e)
 
     @property
     def collection(self):
@@ -442,6 +485,7 @@ class MongoResource(Resource):
             # special case for 'fields' option - fields need to be passed to
             # collection#find
             fields = options.pop('fields', None)
+
             cursor = self.collection.find(self.queryify(query), fields=fields)
 
             # treat options as chaining cursor operations
@@ -500,19 +544,29 @@ class FileResource(Resource):
 
     def read(self, query, options=None):
         with open(self.resolve_path(query)) as fin:
-            file_contents = fin.read()
-            return self.deserialize(file_contents)
+            try:
+                file_contents = fin.read()
+                return self.deserialize(file_contents)
+            except IOError, e:
+                raise exceptions.ConnectionError(e)
 
     def create(self, query, attributes, options=None):
         with open(self.resolve_path(query), 'w') as f:
-            f.write(self.serialize(attributes))
-            return {}
+            try:
+                f.write(self.serialize(attributes))
+                return {}
+            except IOError, e:
+                raise exceptions.ConnectionError(e)
+
     def update(self, query, attributes, options=None):
         self.create(query, attributes, options)
         return attributes
 
     def delete(self, query, options):
-        os.remove(self.resolve_path(query))
+        try:
+            os.remove(self.resolve_path(query))
+        except OSError, e:
+            raise exceptions.ConnectionError(e)
 
     def resolve_path(self, query):
         if '%' in self.path:
